﻿using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Widgets;

/// @author Oskari Eronen
/// @version 1.2.0
///
/// <summary>
/// Pomppupeli, jossa pelaajan tulisi kerätä mahdollisimman paljon renkaita ja ansaita näin pisteitä
/// </summary>
public class Pomppupeli : PhysicsGame
{
    private const double nopeus = 250;
    private const double hyppynopeus = 600;
    private const int ruudunKoko = 40;
    private int kenttaNro = 1;
    private PlatformCharacter pelaaja; // Esitellään pelaaja
    private Image palloKuva = LoadImage("pallo.png"); // Ladataan kuva pallosta
    private Image rengasKuva = LoadImage("rengas.png"); // Ladataan kuva renkaasta
    private Image piikkiKuva = LoadImage("piikki.png"); // Ladataan kuva piikistä
    private Image maaliKuva = LoadImage("maali.png"); // Ladataan kuva maalista
    private IntMeter pisteLaskuri; // Esitellään pistelaskuri
    private List<Label> valikonKohdat; // Esitellään valikonkohdat lista
    private EasyHighScore topLista = new EasyHighScore(); // Esitellään pisteiden top lista
    private List<int> pisteetMaasta; // Esitetään lista jossa pisteet per maa
    private List<Label> maidenPisteet; // Esitetään lopullisen pistevalikon pisteet per maa


    /// <summary>
    /// Pääohjelma luo painovoiman ja aloittaa pelin
    /// </summary>
    public override void Begin()
    {
        Gravity = new Vector(0, -1000);
        Mouse.IsCursorVisible = true;
        pisteetMaasta = new List<int>();
        Valikko(null);
    }


    /// <summary>
    /// Aliohjelma tyhjentää alkuvalikon ja luo kentän
    /// </summary>
    public void AloitaPeli()
    {
        ClearAll();
        kenttaNro = 1;
        pisteetMaasta.Clear();
        LuoKentta("kentta" + kenttaNro);
    }


    /// <summary>
    /// Aliohjelma luo valikon painikkeet ja kuuntelee mitä pelaaja painaa valikossa
    /// </summary>
    /// <param name="sender"> Parametri liittyen toplistan toimintaan </param>
    public void Valikko(Window sender)
    {
        valikonKohdat = new List<Label>();

        Label kohta1 = new Label("Aloita uusi peli");  
        kohta1.Position = new Vector(0, 40);
        valikonKohdat.Add(kohta1);

        Label kohta2 = new Label("Lopeta peli");
        kohta2.Position = new Vector(0, -40);
        valikonKohdat.Add(kohta2);

        Label kohta3 = new Label("Katso pisteet");
        kohta3.Position = new Vector(0, 0);
        valikonKohdat.Add(kohta3);
        
        
        foreach (Label valikonKohta in valikonKohdat) // silmukka lisää jokaisen valikon kohdan 
        {
            Add(valikonKohta);
        }

        Mouse.ListenOn(kohta1, MouseButton.Left, ButtonState.Pressed, AloitaPeli, null);
        Mouse.ListenOn(kohta2, MouseButton.Left, ButtonState.Pressed, Exit, null);
        Mouse.ListenOn(kohta3, MouseButton.Left, ButtonState.Pressed, NaytaTopLista, null);
    }


    /// <summary>
    /// Aliohjelma joka luo kentän tekstitiedostosta
    /// </summary>
    /// <param name="kentannumero"> kentän tekstitiedoston nimi </param>
    public void LuoKentta(string kentannumero)
    {
        ClearAll();

        TileMap kentta = TileMap.FromLevelAsset(kentannumero);
        kentta.SetTileMethod('#', LisaaTaso);
        kentta.SetTileMethod('*', LisaaRengas);
        kentta.SetTileMethod('N', LisaaPelaaja);
        kentta.SetTileMethod('X', LisaaPiikki);
        kentta.SetTileMethod('M', LisaaMaali);
        kentta.Execute(ruudunKoko, ruudunKoko);
        Level.CreateBorders();
        Level.Background.CreateGradient(Color.White, Color.SkyBlue);

        LisaaNappaimet();
        LuoPistelaskuri();

        Camera.Follow(pelaaja);
        Camera.ZoomFactor = 2.0;
        Camera.StayInLevel = true;
    }


    /// <summary>
    /// Aliohjelma tyhjentää edellisen kentän ja kutsuu toista aliohjelmaa kentän tekstitiedoston nimellä
    /// </summary>
    public void SeuraavaKentta()
    {
        LuoKentta("kentta" + kenttaNro);
    }


    /// <summary>
    /// Aliohjelma lisää tason peliin
    /// </summary>
    /// <param name="paikka"> määrittelee mihin taso luodaan </param>
    /// <param name="leveys"> tason leveys </param>
    /// <param name="korkeus"> tason korkeus </param>
    public void LisaaTaso(Vector paikka, double leveys, double korkeus)
    {
        PhysicsObject taso = PhysicsObject.CreateStaticObject(leveys, korkeus);
        taso.Position = paikka;
        taso.Color = Color.DarkRed;
        Add(taso);
    }


    /// <summary>
    /// Alihojelma lisää renkaan peliin
    /// </summary>
    /// <param name="paikka"> määrittelee mihin rengas luodaan </param>
    /// <param name="leveys"> renkaan leveys </param>
    /// <param name="korkeus"> renkaan korkeus </param>
    public void LisaaRengas(Vector paikka, double leveys, double korkeus)
    {
        PhysicsObject rengas = PhysicsObject.CreateStaticObject(leveys, korkeus);
        rengas.IgnoresCollisionResponse = true;
        rengas.Position = paikka;
        rengas.Image = rengasKuva;
        rengas.Tag = "rengas";
        Add(rengas);
    }


    /// <summary>
    /// Aliohjelma lisää piikin peliin
    /// </summary>
    /// <param name="paikka"> määrittelee mihin piikki luodaan </param>
    /// <param name="leveys"> piikin leveys </param>
    /// <param name="korkeus"> piikin korkeus </param>
    public void LisaaPiikki(Vector paikka, double leveys, double korkeus)
    {
        PhysicsObject piikki = PhysicsObject.CreateStaticObject(leveys, korkeus);
        piikki.IgnoresCollisionResponse = true;
        piikki.Position = paikka;
        piikki.Image = piikkiKuva;
        piikki.Tag = "piikki";
        Add(piikki);

    }


    /// <summary>
    /// Alihojelma lisää pelaajan peliin
    /// </summary>
    /// <param name="paikka"> määrittelee mihin pelaaja lisätään </param>
    /// <param name="leveys"> pelaajan leveys </param>
    /// <param name="korkeus"> pelaajan korkeus </param>
    public void LisaaPelaaja(Vector paikka, double leveys, double korkeus)
    {
        pelaaja = new PlatformCharacter(leveys, korkeus);
        pelaaja.Position = paikka;
        pelaaja.Mass = 4.0;
        pelaaja.Image = palloKuva;
        AddCollisionHandler(pelaaja, "rengas", TormaaRenkaaseen);
        AddCollisionHandler(pelaaja, "piikki", TormaaPiikkiin);
        AddCollisionHandler(pelaaja, "maali", TormaaMaaliin);
        Add(pelaaja);
    }


    /// <summary>
    /// Aliohjelma lisää maalin peliin
    /// </summary>
    /// <param name="paikka"> määrittelee mihin maali lisätään </param>
    /// <param name="leveys"> maalin leveys </param>
    /// <param name="korkeus"> maalin korkeus </param>
    public void LisaaMaali(Vector paikka, double leveys, double korkeus)
    {
        PhysicsObject maali = PhysicsObject.CreateStaticObject(leveys, korkeus);
        maali.IgnoresCollisionResponse = true;
        maali.Position = paikka;
        maali.Image = maaliKuva;
        maali.Tag = "maali";
        Add(maali);
    }


    /// <summary>
    /// Aliohjelma lisää peliin näppäimet jolla pelaajaa liikutetaan
    /// </summary>
    public void LisaaNappaimet()
    {
        Keyboard.Listen(Key.O, ButtonState.Pressed, ShowControlHelp, "Näytä ohjeet");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");

        Keyboard.Listen(Key.Left, ButtonState.Down, Liikuta, "Liikkuu vasemmalle", pelaaja, -nopeus);
        Keyboard.Listen(Key.Right, ButtonState.Down, Liikuta, "Liikkuu vasemmalle", pelaaja, nopeus);
        Keyboard.Listen(Key.Up, ButtonState.Pressed, Hyppaa, "Pelaaja hyppää", pelaaja, hyppynopeus);

        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
    }


    /// <summary>
    /// Aliohjelma suorittaa liikkumisen
    /// </summary>
    /// <param name="hahmo"> mitä liikutetaan </param>
    /// <param name="nopeus"> millä nopeudella liikutetaan </param>
    public void Liikuta(PlatformCharacter hahmo, double nopeus)
    {
        hahmo.Walk(nopeus);
    }


    /// <summary>
    /// Aliohjelma suorittaa hyppäämisen
    /// </summary>
    /// <param name="hahmo"> millä hypätään </param>
    /// <param name="nopeus"> millä nopeudella hypätään eli kuinka korkealle </param>
    public void Hyppaa(PlatformCharacter hahmo, double nopeus)
    {
        hahmo.Jump(nopeus);
    }


    /// <summary>
    /// Aliohjelma suorittaa tapahtumat kun pelaaja törmää renkaaseen
    /// </summary>
    /// <param name="hahmo"> mikä törmää </param>
    /// <param name="rengas"> mihin törmätään </param>
    public void TormaaRenkaaseen(PhysicsObject hahmo, PhysicsObject rengas)
    {
        MessageDisplay.Add("Keräsit renkaan!");
        pisteLaskuri.Value += 1;
        rengas.Destroy();
    }


    /// <summary>
    /// Aliohjelma suorittaa tapahtumat kun pelaaja törmää piikkiin
    /// </summary>
    /// <param name="hahmo"> mikä törmää </param>
    /// <param name="piikki"> mihin törmätään </param>
    public void TormaaPiikkiin(PhysicsObject hahmo, PhysicsObject piikki)
    {
        MessageDisplay.Add("Pallo puhkesi, voi ei!");
        hahmo.Destroy();
        Kuolema();

    }


    /// <summary>
    /// Aliohjelma suorittaa tapahtumat kun pelaaja törmää maaliin
    /// </summary>
    /// <param name="hahmo"> mikä törmää </param>
    /// <param name="maali"> mihin törmätään </param>
    public void TormaaMaaliin(PhysicsObject hahmo, PhysicsObject maali)
    {
        kenttaNro++;
        pisteetMaasta.Add(pisteLaskuri.Value);
        if (kenttaNro < 5)
        {
            SeuraavaKentta();
        }
        else
        {
            LopullisetPisteet();
        }
        
    }


    /// <summary>
    /// Alihojelma suorittaa tapahtumat kun pelaaja on törmännyt piikkiin ja kuolee
    /// </summary>
    public void Kuolema()
    {
        ClearAll();
        topLista.EnterAndShow(KokonaisPisteet(pisteetMaasta));
        topLista.HighScoreWindow.Closed += Valikko;
    }


    /// <summary>
    /// Aliohjelma luo pistelaskurin peliin
    /// </summary>
    public void LuoPistelaskuri()
    {
        pisteLaskuri = new IntMeter(0);

        Label pisteNaytto = new Label();
        pisteNaytto.X = Screen.Right - 150;
        pisteNaytto.Y = Screen.Top - 100;
        pisteNaytto.TextColor = Color.Black;
        pisteNaytto.Color = Color.White;
        pisteNaytto.Title = "Tämän maan pisteet: ";

        pisteNaytto.BindTo(pisteLaskuri);
        Add(pisteNaytto);
    }


    /// <summary>
    /// Aliohjelma näyttää kerätyt pisteet per maa
    /// </summary>
    public void LopullisetPisteet()
    {
        ClearAll();

        maidenPisteet = new List<Label>();

        Label maa1 = new Label("Pisteet maasta 1: " + pisteetMaasta[0]);
        maa1.Position = new Vector(0, 40);
        maidenPisteet.Add(maa1);

        Label maa2 = new Label("Pisteet maasta 2: " + pisteetMaasta[1]);
        maa2.Position = new Vector(0, 20);
        maidenPisteet.Add(maa2);

        Label maa3 = new Label("Pisteet maasta 3: " + pisteetMaasta[2]);
        maa3.Position = new Vector(0, 0);
        maidenPisteet.Add(maa3);

        Label maa4 = new Label("Pisteet maasta 4: " + pisteetMaasta[3]);
        maa4.Position = new Vector(0, -20);
        maidenPisteet.Add(maa4);

        Label yhteensa = new Label("YHTEENSÄ: " + KokonaisPisteet(pisteetMaasta));
        yhteensa.Position = new Vector(0, -40);
        maidenPisteet.Add(yhteensa);

        Label poistu = new Label("Poistu Valikkoon");
        poistu.Position = new Vector(0, -60);
        maidenPisteet.Add(poistu);


        foreach (Label pisteet in maidenPisteet) // silmukka lisää jokaisen valikon kohdan 
        {
            pisteet.Color = Color.White;
            Add(pisteet);
        }

        Mouse.ListenOn(poistu, MouseButton.Left, ButtonState.Pressed, Lopetus, null);
    }


    /// <summary>
    /// Aliohjelmassa lasketaan yhteen listaan lisätyt arvot
    /// </summary>
    /// <param name="pisteet"> lista josta pisteet lasketaan </param>
    /// <returns></returns>
    public static int KokonaisPisteet(List<int> pisteet)
    {
        int kokonaispisteet = 0;
        for (int i = 0; i < pisteet.Count; i++)
        {
            kokonaispisteet += pisteet[i];
        }
        return kokonaispisteet;
    }


    /// <summary>
    /// Aliohjelma näyttää pisteiden TOP listan
    /// </summary>
    public void NaytaTopLista()
    {
        ClearAll();
        topLista.Show();
        topLista.HighScoreWindow.Closed += Valikko;
    }


    /// <summary>
    /// Aliohjelma pyytää lisämään lopulliset pisteet TOP listaan
    /// </summary>
    public void Lopetus()
    {
        ClearAll();
        topLista.EnterAndShow(KokonaisPisteet(pisteetMaasta));
        topLista.HighScoreWindow.Closed += Valikko;
    }
}


